@@ -9,6 +9,7 @@
 #include <haproxy/mux_quic.h>
 #include <haproxy/pool.h>
 #include <haproxy/quic_conn.h>
+#include <haproxy/quic_utils.h>
 #include <haproxy/task.h>
 
 DECLARE_STATIC_POOL(pool_head_quic_stream_desc, "qc_stream_desc",
@@ -45,6 +46,8 @@ static void qc_stream_buf_free(struct qc_stream_desc *stream,
 		pool_free(pool_head_sbuf, buf->area);
 	}
 	else {
+		bdata_ctr_del(&stream->data, b_data(buf));
+		bdata_ctr_bdec(&stream->data);
 		b_free(buf);
 		offer_buffers(NULL, 1);
 	}
@@ -77,14 +80,15 @@ struct qc_stream_desc *qc_stream_desc_new(uint64_t id, enum qcs_type type, void
 	else {
 		stream->by_id.key = id;
 		eb64_insert(&qc->streams_by_id, &stream->by_id);
-		qc->rx.strms[type].nb_streams++;
 	}
 	stream->qc = qc;
 
 	stream->buf = NULL;
 	stream->buf_tree = EB_ROOT_UNIQUE;
 	stream->buf_offset = 0;
+	bdata_ctr_init(&stream->data);
 
+	stream->origin_ts = now_ns;
 	stream->ack_offset = 0;
 	stream->flags = 0;
 	stream->ctx = ctx;
@@ -128,7 +132,7 @@ void qc_stream_desc_release(struct qc_stream_desc *stream,
 		if (final_size < tail_offset)
 			b_sub(buf, tail_offset - final_size);
 
-		/* Release active buffer, or delete it immediatly if there is
+		/* Release active buffer, or delete it immediately if there is
 		 * no data to acknowledge. Both functions will reset active
 		 * buf pointer and invoke <notify_room> if necessary.
 		 */
@@ -243,7 +247,7 @@ static int qc_stream_buf_store_ack(struct qc_stream_buf *buf,
 }
 
 /* Acknowledges data for buffer <buf> attached to <stream> instance. This covers
- * the range strating at <offset> and of length <len>, with <fin> sets for the
+ * the range starting at <offset> and of length <len>, with <fin> sets for the
  * last stream frame.
  *
  * Returns <buf> if there is still data to acknowledge or buffered ACK to
@@ -263,6 +267,7 @@ static struct qc_stream_buf *qc_stream_buf_ack(struct qc_stream_buf *buf,
 		diff = offset + len - stream->ack_offset;
 		b_del(&buf->buf, diff);
 		stream->ack_offset += diff;
+		bdata_ctr_del(&stream->data, diff);
 
 		/* notify room from acked data if buffer has been released. */
 		if (stream->notify_room && qc_stream_buf_is_released(buf, stream)) {
@@ -481,6 +486,7 @@ struct buffer *qc_stream_buf_alloc(struct qc_stream_desc *stream,
 	}
 
 	eb64_insert(&stream->buf_tree, &stream->buf->offset_node);
+	bdata_ctr_binc(&stream->data);
 
 	return &stream->buf->buf;
 }
@@ -495,7 +501,7 @@ struct buffer *qc_stream_buf_realloc(struct qc_stream_desc *stream)
 	/* This function is reserved to convert a big buffer to a smaller one. */
 	BUG_ON(!stream->buf || !stream->buf->sbuf);
 
-	/* This function can only be used if targetted buffer is empty. */
+	/* This function can only be used if targeted buffer is empty. */
 	BUG_ON(b_data(&stream->buf->buf));
 
 	/* Release buffer */
