@@ -141,11 +141,14 @@ static SSL_CTX *ssl_sock_do_create_cert(const char *servername, struct bind_conf
 
 	/* Set the subject name using the same, but the CN */
 	name = X509_NAME_dup(name);
-	if (X509_NAME_add_entry_by_txt(name, "CN", MBSTRING_ASC,
-				       (const unsigned char *)servername,
-				       -1, -1, 0) != 1) {
-		X509_NAME_free(name);
-		goto mkcert_error;
+
+	if (strlen(servername) <= 64) {
+		if (X509_NAME_add_entry_by_txt(name, "CN", MBSTRING_ASC,
+		                              (const unsigned char *)servername,
+		                              -1, -1, 0) != 1) {
+			X509_NAME_free(name);
+			goto mkcert_error;
+		}
 	}
 	if (X509_set_subject_name(newcrt, name) != 1) {
 		X509_NAME_free(name);
@@ -352,6 +355,8 @@ int ssl_sock_generate_certificate(const char *servername, struct bind_conf *bind
 			ssl_ctx = (SSL_CTX *)lru->data;
 		if (!ssl_ctx && lru) {
 			ssl_ctx = ssl_sock_do_create_cert(servername, bind_conf, ssl);
+			if (!ssl_ctx)
+				goto error;
 			lru64_commit(lru, ssl_ctx, cacert, 0, (void (*)(void *))SSL_CTX_free);
 		}
 		SSL_set_SSL_CTX(ssl, ssl_ctx);
@@ -360,11 +365,14 @@ int ssl_sock_generate_certificate(const char *servername, struct bind_conf *bind
 	}
 	else {
 		ssl_ctx = ssl_sock_do_create_cert(servername, bind_conf, ssl);
+		if (!ssl_ctx)
+			goto error;
 		SSL_set_SSL_CTX(ssl, ssl_ctx);
 		/* No LRU cache, this CTX will be released as soon as the session dies */
 		SSL_CTX_free(ssl_ctx);
 		return 1;
 	}
+error:
 	return 0;
 }
 int ssl_sock_generate_certificate_from_conn(struct bind_conf *bind_conf, SSL *ssl)
